use crate::service::error::Error;
use axum::response::Html;
use minijinja::{path_loader, Environment, Template, Value};
use minijinja_autoreload::AutoReloader;
use pulldown_cmark::{Event, Tag};
use std::sync::Arc;

pub struct RenderTarget {}

#[derive(Clone)]
pub struct Templates {
    pub reloader: Arc<minijinja_autoreload::AutoReloader>,
}

impl Templates {
    pub fn initialize() -> Result<Self, String> {
        let reloader = AutoReloader::new(move |notifier| {
            let mut environment = Environment::new();
            let template_path = "templates";
            //environment.set_loader();

            environment.set_loader(path_loader(&template_path));
            environment.add_filter("markdown", markdown);

            notifier.watch_path(template_path, true);
            //environment.set_source(Source::from_path(template_path));
            Ok(environment)
        });
        Ok(Self {
            reloader: Arc::new(reloader),
        })
    }

    pub fn render<D: serde::Serialize>(&self, key: &str, data: D) -> Result<String, Error> {
        let env = self.reloader.acquire_env()?;
        let template = env.get_template(key)?;
        let rendered = template.render(&data)?;

        Ok(rendered)
    }

    pub fn render_html<D: serde::Serialize>(
        &self,
        key: &str,
        data: D,
    ) -> Result<Html<String>, Error> {
        let result = self.render(key, data)?;
        Ok(Html(result))
    }

    pub fn render_indirect(&self, key: &str) -> Result<RenderTarget, Error> {
        let env = self.reloader.acquire_env()?;

        Ok(RenderTarget {})
    }
}

fn markdown(value: String) -> Value {
    let parser = pulldown_cmark::Parser::new(&value);
    let parser: Box<dyn Iterator<Item = Event>> = Box::new(parser.into_iter());
    let mut html = String::new();
    pulldown_cmark::html::push_html(&mut html, parser);
    Value::from_safe_string(html)
    /*
    value
        .to_lowercase()
        .split_whitespace()
        .collect::<Vec<_>>()
        .join("-")
     */
}
